package org.eclipse.gmt.am3.platform.extension.gmm4sa.ui.editors.pages;

import java.util.Collection;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.gmt.am3.core.modeling.ModelElement;
import org.eclipse.gmt.am3.platform.core.AM3Platform;
import org.eclipse.gmt.am3.platform.runtime.core.context.AM3Context;
import org.eclipse.gmt.am3.platform.ui.dialogs.ModelElementSelectionDialog;
import org.eclipse.gmt.am3.platform.ui.editors.MegamodelEditor;
import org.eclipse.gmt.am3.platform.ui.editors.MegamodelEditorInput;
import org.eclipse.gmt.am3.platform.ui.editors.pages.AM3AbstractPage;
import org.eclipse.gmt.am3.platform.ui.views.parts.AM3StandardViewSection;
import org.eclipse.gmt.am3.platform.ui.views.parts.FeatureSection;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.forms.widgets.Section;
 
public class ArchitectureModel extends AM3AbstractPage {
	protected AM3StandardViewSection viewsSection;
	protected AM3StandardViewSection decisionsSection;

	protected void createFormContent(IManagedForm managedForm) {
		super.createFormContent(managedForm);
		refreshEditorTitle();
	}

	private void refreshEditorTitle() {
		AM3Context context= AM3Context.getDefault();
		Object value = context.getModelUtil().getFeatureValue(modelElement, "identifier",
				"value");
		// Editor Title
		if (value != null) {
			((MegamodelEditor) getEditor()).setEditorTitle(modelElement
					.getMetaElement().getFullyQualifiedName()
					+ " [" + value.toString() + "]");
		}
	}

	protected void addControls(Composite parent) {
		referenceModelSection = toolkit.createSection(parent, Section.TITLE_BAR | Section.EXPANDED);
		referenceModelSection.setText("Conforms To");
		GridData rmData = new GridData(GridData.FILL_HORIZONTAL);
		rmData.horizontalSpan = 2;
		referenceModelSection.setLayoutData(rmData);
		Composite referenceModelSectionClient = toolkit.createComposite(referenceModelSection);
		referenceModelSection.setClient(referenceModelSectionClient);
		referenceModelSectionClient.setLayout(new GridLayout(3, false));
		Hyperlink h = toolkit.createHyperlink(referenceModelSectionClient, "Model kind:", SWT.None);
		h.addHyperlinkListener(new HyperlinkAdapter() {
			public void linkActivated(HyperlinkEvent e) {
				ModelElement conformsTo = (ModelElement) modelElement.get("conformsTo");
				if(conformsTo != null) {
					IEditorInput editorInput = new MegamodelEditorInput("MegamodelEditorInput", conformsTo);
					IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
					try {
						page.openEditor(editorInput, MegamodelEditor.ID);
					} catch (PartInitException e1) {
						e1.printStackTrace();
					}
				}
			}
		});
		referenceModelText = toolkit.createText(referenceModelSectionClient, "");
		referenceModelText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		initReferenceModelText();
		referenceModelBrowseButton = toolkit.createButton(referenceModelSectionClient, "Browse...", SWT.None);
		referenceModelBrowseButton.addSelectionListener(new IdentifierBrowseSelection());

		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;

		// End
		/*
		 * sourceOf Section
		 */
		viewsSection = new FeatureSection(
				modelElement,
				modelElement.getMetaElement().getFeature(
						"views"),
				toolkit, parent);
		viewsSection.createControls();
		// end

		/*
		 * Related To Section
		 */
		decisionsSection = new FeatureSection(
				modelElement,
				modelElement
						.getMetaElement()
						.getFeature(
								"decisions"),
				toolkit, parent);

		data = new GridData(GridData.FILL_BOTH);
		//data.horizontalSpan = 2;
		decisionsSection.createControls();
		decisionsSection.getControl().setLayoutData(data);
		/*
		 * end
		 */
		viewsSection.refresh();
		decisionsSection.refresh();
	};


	public void am3Changed() {
		viewsSection.refresh();
		decisionsSection.refresh();
		refreshEditorTitle();
	}
	
	
	private ModelElement currentlySelectedReferenceModel;
	
	private Section referenceModelSection;
	private Text referenceModelText;
	private Button referenceModelBrowseButton;
	
	class IdentifierBrowseSelection extends SelectionAdapter {
		@Override
		public void widgetSelected(SelectionEvent e) {
			AM3Context context= AM3Context.getDefault();
			ModelElementSelectionDialog dialog = new ModelElementSelectionDialog(
					referenceModelSection.getShell());
			dialog.setText("Select the Model Kind");
			String metaElementName;
			if( modelElement.isKindOf(AM3Platform.getAM3Metamodel().getMetaElementByName("GlobalModelManagement::TerminalModel")) )
				metaElementName = "GlobalModelManagement::Metamodel";
			else
				metaElementName = "GlobalModelManagement::Metametamodel";
			Collection<ModelElement> referenceModels = context.getModelUtil().getAllElements(AM3Platform.getCurrentMegaModel(), metaElementName); 
			dialog.setViewerInput(referenceModels);
			if (dialog.open() == Dialog.OK) {
				ModelElement referenceModel = dialog.getSelectionElement();
				ModelElement identifier = (ModelElement) referenceModel.get("identifier");
				String newId = (String)identifier.get("value");
				if( !newId.equals(referenceModelText.getText()) ) {
					referenceModelText.setText(newId);
					currentlySelectedReferenceModel = referenceModel;
					makeDirty();
				}
			}
		}
	}
	
	private void initReferenceModelText() {
		ModelElement referenceModel = (ModelElement) modelElement.get("conformsTo");
		if( referenceModel!=null )  {
			ModelElement identifier = (ModelElement) referenceModel.get("identifier");
			referenceModelText.setText((String)identifier.get("value"));
			currentlySelectedReferenceModel = referenceModel;
		}
	}
	
	@Override
	public void doSave(IProgressMonitor monitor) {		
		ModelElement atlModule = (ModelElement) modelElement.get("conformsTo");
		if( atlModule!=this.currentlySelectedReferenceModel )
			modelElement.set("conformsTo", this.currentlySelectedReferenceModel);
		
		super.doSave(monitor);
	}
	
//	private String getCurrentReferenceModelId() {
//		String id = "";
//		ModelElement referenceModel = (ModelElement) modelElement.get("conformsTo");
//		if( referenceModel!=null )  {
//			currentlySelectedReferenceModel = referenceModel;
//			ModelElement identifier = (ModelElement) referenceModel.get("identifier");
//			id = (String)identifier.get("value");
//		}
//		return id;
//	}
	
	
}
